@@ -23,6 +23,8 @@ othptab.c - non-TCP protocol display module
 #include "servname.h"
 #include "addproto.h"
 #include "packet.h"
+#include "hostmon.h"
+#include "sockaddr.h"
 
 #define MSGSTRING_MAX	240
 #define SHORTSTRING_MAX	40
@@ -33,8 +35,6 @@ othptab.c - non-TCP protocol display module
 */
 #define uninitialized_var(x) x = x
 
-void convmacaddr(char *addr, char *result);	/* external; from hostmon.c */
-
 static void writeothplog(int logging, FILE *fd, char *protname,
 			 char *description, char *additional, int is_ip,
 			 int withmac, struct othptabent *entry)
@@ -86,7 +86,7 @@ static void writeothplog(int logging, FILE *fd, char *protname,
 	}
 }
 
-void init_othp_table(struct othptable *table, int mac)
+void init_othp_table(struct othptable *table)
 {
 	unsigned int winht;
 	unsigned int wintop;
@@ -117,8 +117,6 @@ void init_othp_table(struct othptable *table, int mac)
 	tx_stdwinset(table->othpwin);
 	getmaxyx(table->borderwin, table->obmaxy, obmaxx);
 	table->oimaxy = table->obmaxy - 2;
-
-	table->mac = mac;
 }
 
 void process_dest_unreach(struct tcptable *table, char *packet, char *ifname)
@@ -131,10 +129,8 @@ void process_dest_unreach(struct tcptable *table, char *packet, char *ifname)
 	ip = (struct iphdr *) (packet + 8);
 
 	/*
-	 * Timeout checking
-	 * won't be performed either, so we just pass NULL as the pointer
-	 * to the configuration structure.  in_table() will recognize this
-	 * and set its internal timeout variable to 0.
+	 * Timeout checking won't be performed either, so we just pass 0
+	 * as timeout variable.
 	 */
 
 	if (ip->version == 6) {
@@ -142,18 +138,24 @@ void process_dest_unreach(struct tcptable *table, char *packet, char *ifname)
 		if (ip6->ip6_nxt != IPPROTO_TCP)
 			return;
 		tcp = (struct tcphdr *) (packet + 48);
+		struct sockaddr_storage saddr, daddr;
+		sockaddr_make_ipv6(&saddr, &ip6->ip6_src);
+		sockaddr_set_port(&saddr, ntohs(tcp->source));
+		sockaddr_make_ipv6(&daddr, &ip6->ip6_dst);
+		sockaddr_set_port(&daddr, ntohs(tcp->dest));
 		tcpentry =
-		    in_table(table, 0, 0, ip6->ip6_src.s6_addr,
-			     ip6->ip6_dst.s6_addr, ntohs(tcp->source),
-			     ntohs(tcp->dest), ifname, 0, NULL, NULL);
+		    in_table(table, &saddr, &daddr, ifname, 0, NULL, 0);
 	} else {
 		if (ip->protocol != IPPROTO_TCP)
 			return;
 		tcp = (struct tcphdr *) (packet + 8 + (ip->ihl * 4));
+		struct sockaddr_storage saddr, daddr;
+		sockaddr_make_ipv4(&saddr, ip->saddr);
+		sockaddr_set_port(&saddr, ntohs(tcp->source));
+		sockaddr_make_ipv4(&daddr, ip->daddr);
+		sockaddr_set_port(&daddr, ntohs(tcp->dest));
 		tcpentry =
-		    in_table(table, ip->saddr, ip->daddr, NULL, NULL,
-			     ntohs(tcp->source), ntohs(tcp->dest), ifname, 0,
-			     NULL, NULL);
+		    in_table(table, &saddr, &daddr, ifname, 0, NULL, 0);
 	}
 
 	if (tcpentry != NULL) {
@@ -163,61 +165,40 @@ void process_dest_unreach(struct tcptable *table, char *packet, char *ifname)
 }
 
 struct othptabent *add_othp_entry(struct othptable *table, struct pkt_hdr *pkt,
-				  unsigned long saddr,
-				  unsigned long daddr, struct in6_addr *s6addr,
-				  struct in6_addr *d6addr, int is_ip,
+				  struct sockaddr_storage *saddr,
+				  struct sockaddr_storage *daddr,
+				  int is_ip,
 				  int protocol,
 				  char *packet2,
 				  char *ifname, int *rev_lookup, int rvnfd,
-				  int logging, FILE * logfile,
-				  int servnames, int fragment)
+				  int logging, FILE *logfile, int fragment)
 {
 	struct othptabent *new_entry;
 	struct othptabent *temp;
-	struct in_addr isaddr, idaddr;
-	char *packet = pkt->pkt_buf;
 
 	new_entry = xmallocz(sizeof(struct othptabent));
 
 	new_entry->is_ip = is_ip;
 	new_entry->fragment = fragment;
 
-	if ((table->mac) || (!is_ip)) {
+	if (options.mac || !is_ip) {
 		if (pkt->pkt_hatype == ARPHRD_ETHER) {
-			convmacaddr((char *) (((struct ethhdr *) packet)->
-					      h_source), new_entry->smacaddr);
-			convmacaddr((char *) (((struct ethhdr *) packet)->
-					      h_dest), new_entry->dmacaddr);
+			convmacaddr((char *) pkt->ethhdr->h_source, new_entry->smacaddr);
+			convmacaddr((char *) pkt->ethhdr->h_dest, new_entry->dmacaddr);
 		} else if (pkt->pkt_hatype == ARPHRD_FDDI) {
-			convmacaddr((char *) (((struct fddihdr *) packet)->
-					      saddr), new_entry->smacaddr);
-			convmacaddr((char *) (((struct fddihdr *) packet)->
-					      daddr), new_entry->dmacaddr);
-		} else if ((pkt->pkt_hatype == ARPHRD_IEEE802)
-			   || (pkt->pkt_hatype == ARPHRD_IEEE802_TR)) {
-			convmacaddr((char *) (((struct trh_hdr *) packet)->
-					      saddr), new_entry->smacaddr);
-			convmacaddr((char *) (((struct trh_hdr *) packet)->
-					      daddr), new_entry->dmacaddr);
+			convmacaddr((char *) pkt->fddihdr->saddr, new_entry->smacaddr);
+			convmacaddr((char *) pkt->fddihdr->daddr, new_entry->dmacaddr);
 		}
 	}
 
 	if (is_ip) {
-		new_entry->saddr = isaddr.s_addr = saddr;
-		new_entry->daddr = idaddr.s_addr = daddr;
+		sockaddr_copy(&new_entry->saddr, saddr);
+		sockaddr_copy(&new_entry->daddr, daddr);
 
-		if (s6addr != NULL)
-			memcpy(&new_entry->s6addr, s6addr, 16);
-		else
-			memset(&new_entry->s6addr, 0, 16);
-
-		if (d6addr != NULL)
-			memcpy(&new_entry->d6addr, d6addr, 16);
-		else
-			memset(&new_entry->s6addr, 0, 16);
-
-		revname(rev_lookup, &isaddr, s6addr, new_entry->s_fqdn, rvnfd);
-		revname(rev_lookup, &idaddr, d6addr, new_entry->d_fqdn, rvnfd);
+		revname(rev_lookup, saddr, new_entry->s_fqdn,
+			sizeof(new_entry->s_fqdn), rvnfd);
+		revname(rev_lookup, daddr, new_entry->d_fqdn,
+			sizeof(new_entry->d_fqdn), rvnfd);
 
 		if (!fragment) {
 			if (protocol == IPPROTO_ICMP) {
@@ -231,12 +212,10 @@ struct othptabent *add_othp_entry(struct othptable *table, struct pkt_hdr *pkt,
 				new_entry->un.icmp6.code =
 				    ((struct icmp6_hdr *) packet2)->icmp6_code;
 			} else if (protocol == IPPROTO_UDP) {
-				servlook(servnames,
-					 ((struct udphdr *) packet2)->source,
+				servlook(ntohs(((struct udphdr *) packet2)->source),
 					 IPPROTO_UDP, new_entry->un.udp.s_sname,
 					 10);
-				servlook(servnames,
-					 ((struct udphdr *) packet2)->dest,
+				servlook(ntohs(((struct udphdr *) packet2)->dest),
 					 IPPROTO_UDP, new_entry->un.udp.d_sname,
 					 10);
 			} else if (protocol == IPPROTO_OSPFIGP) {
@@ -245,9 +224,10 @@ struct othptabent *add_othp_entry(struct othptable *table, struct pkt_hdr *pkt,
 				new_entry->un.ospf.area =
 				    ntohl(((struct ospfhdr *) packet2)->
 					  ospf_areaid.s_addr);
-				strcpy(new_entry->un.ospf.routerid,
-				       inet_ntoa(((struct ospfhdr *)
-						  packet2)->ospf_routerid));
+				inet_ntop(AF_INET,
+					  &((struct ospfhdr *)packet2)->ospf_routerid,
+					  new_entry->un.ospf.routerid,
+					  sizeof(new_entry->un.ospf.routerid));
 			}
 		}
 	} else {
@@ -410,7 +390,7 @@ void printothpentry(struct othptable *table, struct othptabent *entry,
 				break;
 			}
 
-			sprintf(scratchpad, inet_ntoa(saddr));
+			inet_ntop(AF_INET, &saddr, scratchpad, sizeof(scratchpad));
 			strcat(msgstring, scratchpad);
 			wattrset(table->othpwin, ARPATTR);
 			break;
@@ -736,7 +716,7 @@ void printothpentry(struct othptable *table, struct othptabent *entry,
 
 	strcat(msgstring, scratchpad);
 
-	if (((entry->smacaddr)[0] != '\0') && (table->mac)) {
+	if (((entry->smacaddr)[0] != '\0') && options.mac) {
 		snprintf(scratchpad, MSGSTRING_MAX, " (src HWaddr %s)",
 			 entry->smacaddr);
 		strcat(msgstring, scratchpad);
@@ -754,7 +734,7 @@ void printothpentry(struct othptable *table, struct othptabent *entry,
 
 	if (logging)
 		writeothplog(logging, logfile, protname, description,
-			     additional, 1, table->mac, entry);
+			     additional, 1, options.mac, entry);
 }
 
 void refresh_othwindow(struct othptable *table)
