@@ -24,8 +24,7 @@
 struct rvn {
 	int type;
 	int ready;
-	struct in_addr saddr;
-	struct in6_addr s6addr;
+	struct sockaddr_storage addr;
 	char fqdn[45];
 };
 
